package com.emsoft.data.mongo

import com.emsoft.common.Assert
import com.emsoft.common.StringHelper
import com.emsoft.data.SessionDataManager
import com.emsoft.exceptions.ExHelper
import com.gmongo.GMongo
import com.mongodb.ServerAddress

/**
 * @author Vladimir Zhukov
 */
class MongoDataManager extends SessionDataManager<GMongo, MongoDataManager> {

    private String configFile
    private MongoDbOptions options = new MongoDbOptions()

    //    protected MongoDataManager() {
    //    }

    MongoDataManager(String host, int port) {
        Assert.hasText(host, "host")
        options.host = host
        options.port = port
    }

    MongoDataManager(String configFile = "META-INF/mongodb.groovy") {
        Assert.hasText(configFile, "configFile")
        this.configFile = configFile
    }

    MongoDataManager(MongoDataManager dataManager) {
        super(dataManager)
    }

    @Override
    protected boolean isOpenCore() {
        return sessionObject != null && sessionObject.mongo.connector.open
    }

    @Override
    protected boolean openCore() {
        if (StringHelper.hasText(configFile)) {
            def message = StringHelper.EMPTY
            try {
                options.load(configFile)
            }
            catch (ex) {
                message = ExHelper.getMessage(ex)
            }
            if (options == null || !StringHelper.hasText(options.host))
                MongoExHelper.throwWrongOptionsFromConfigFile(message)
        }
        sessionObject = new GMongo(new ServerAddress(options.host, options.port), options.toMongoOptions())
        //session.getDB(options.db).
        return isOpenCore()
    }

    @Override
    protected boolean closeCore() {
        sessionObject.close()
        sessionObject = null
        return !isOpenCore()
    }

    //region

    @Override
    protected boolean rollbackTransactionCore() {
        ExHelper.throwUnsupportedOperation()
        return false
    }

    @Override
    protected boolean commitTransactionCore() {
        ExHelper.throwUnsupportedOperation()
        return false
    }

    @Override
    protected boolean beginTransactionCore() {
        ExHelper.throwUnsupportedOperation()
        return false
    }

    @Override
    protected boolean isInTransactionCore() {
        return false
    }

    //endregion
}
